# Variables for inspecting event dictionaries


PORTAL_RECORD_PK_KEY = 'pgh_obj_id'
OPERATION_KEY = 'operation'
OPERATION_CONFIRM_SYNCED = 'confirm_synced'
SYNCMODE_KEY = 'syncmode'
SYNC_COMPLETED_KEY = 'synced'
