from lcconfig import ConcordiaYmlConfig

from .server import ClarionServer

if __name__ == '__main__':
    cfg = ConcordiaYmlConfig().as_typed_dict()

server = ClarionServer(
    server=cfg['server'],
    user=cfg['user'],
    password=cfg['password'],
    application=cfg['asgi_application'],
)

server.run()
