# Tasks for syncing via REST API


# @shared_task(base=LCSingleton, lock_expiry=43200, raise_on_duplicate=True)
# def synchronize_all(batches, **kwargs):
#    return 'done'


# __all__ = [
#    'synchronize_all',
# ]
