from console_base.api.routers import LCSimpleRouter
from . import views as cv

router = LCSimpleRouter()
router.register(
    r'concordia/canonicaliddeletelog',
    cv.CanonicalIdDeleteLogViewSet,
    basename='api-canonicaliddeletelog'
)
router.register(
    r'concordia/canonicalidmap', cv.CanonicalIdMapViewSet, basename='api-canonicalidmap'
)
router.register(
    r'concordia/canonicalidswap', cv.CanonicalIdSwapViewSet, basename='api-canonicalidswap'
)
